import re
from typing import Any, Dict, List, Union

from openodia.constants import STOPWORDS


class UnderstandData:
    """Tokenizer implementation"""

    @classmethod
    def word_tokenizer(cls, text):
        """Split the text into words"""
        # TODO: Do not remove the punctuation characters
        token_list = [
            token
            for token in re.split(
                r"[!\"#$%&\'()*+,-./:;<=>?@[\\\]\^\_`{|}~୲—\s+]", text, flags=re.UNICODE
            )
            if token
        ]
        return token_list

    @classmethod
    def sentence_tokenizer(cls, text):
        """Split the text into sentences"""
        raise NotImplementedError("Not implemented yet. Will be implemented soon!")

    @classmethod
    def remove_stopwords(cls, text: str, get_str: bool = False) -> Union[List[str], str]:
        """Remove frequently used words from the text"""
        token_list: List[str] = cls.word_tokenizer(text)
        cleaned_tokens = [token for token in token_list if token not in STOPWORDS]
        return " ".join(cleaned_tokens) if get_str else cleaned_tokens

    @classmethod
    def detect_language(cls, text: str, threshold: float = 0.5) -> Dict[str, Any]:
        """Detect language if it is Odia or non-Odia
        :param text: text to detect language
        :param threshold: confidence score which if crossed will be determined as Odia
        """
        if len(text) == 0:
            print("No text detected")
            return {}
        space_removed_text = text.replace(" ", "")
        odia_text = "".join(
            [letter for letter in space_removed_text if ord(letter) in range(2817, 2931)]
        )
        score = len(odia_text) / len(space_removed_text)
        language = "odia" if score > threshold else "non-odia"
        confidence_score = score if language == "odia" else 1 - score
        return {"language": language, "confidence_score": confidence_score}



"""Open Odia language tools
"""
__version__ = "0.0.13"

from .letters import Letters as alphabet
from .odianames import Names as name
from .understandData import UnderstandData as UD

__all__ = ["alphabet", "name", "UD"]



from random import shuffle
from typing import List

from faker import Faker

from openodia.constants import (
    PREFIXES,
    FIRST_NAMES_MALE,
    FIRST_NAMES_FEMALE,
    FIRST_NAMES_UNISEX,
    FIRST_NAMES,
    MIDDLE_NAMES,
    LAST_NAMES,
)


class Names:
    """Names in Odia"""

    @classmethod
    def generate_prefixes(cls, count: int = 10) -> List[str]:
        """Generate prefixes
        :param count: number of prefixes to generate
        """
        shuffle(PREFIXES)
        return PREFIXES[:count]

    @classmethod
    def generate_names(cls, count: int = 10) -> List[str]:
        """Generate Odia names
        :param count: number of names to generate
        """
        fake = Faker("or_IN")
        name_list = [fake.name() for _ in range(count)]
        return name_list

    @classmethod
    def generate_firstnames(cls, count: int = 10, type: str = "") -> List[str]:
        """Generate first names
        :param count: number of names to generate
        :param type: types of name to give
        """
        valid_types = ("female", "male", "unisex")
        if type.lower() == "male":
            shuffle(FIRST_NAMES_MALE)
            return FIRST_NAMES_MALE[:count]
        elif type.lower() == "female":
            shuffle(FIRST_NAMES_FEMALE)
            return FIRST_NAMES_FEMALE[:count]
        elif type.lower() == "unisex":
            shuffle(FIRST_NAMES_UNISEX)
            return FIRST_NAMES_UNISEX[:count]
        elif len(type) > 0:
            print(f"Invalid {type=} provided.\n Please provide one of these {valid_types=}")
        else:
            # send mix of all first names
            shuffle(FIRST_NAMES)
            return FIRST_NAMES[:count]

    @classmethod
    def generate_middlenames(cls, count: int = 10) -> List[str]:
        """Generate middle names
        :param count: number of middle names to generate
        """
        shuffle(MIDDLE_NAMES)
        return MIDDLE_NAMES[:count]

    @classmethod
    def generate_surnames(cls, count: int = 10) -> List[str]:
        """Generate surnames
        :param count: number of surnames to generate
        """
        shuffle(LAST_NAMES)
        return LAST_NAMES[:count]



"""Setup script for openodia
Referred:   https://github.com/realpython/reader/blob/master/setup.py
            https://realpython.com/pypi-publish-python-package/
"""

import os.path
from setuptools import find_packages, setup

# The directory containing this file
HERE = os.path.abspath(os.path.dirname(__file__))

# The text of the README file
with open(os.path.join(HERE, "README.md")) as fid:
    README = fid.read()

# The text of the requirements file
with open(os.path.join(HERE, "requirements.txt")) as rt:
    REQUIREMENTS = rt.readlines()

# This call to setup() does all the work
setup(
    name="openodia",
    version="0.0.13",
    description="Open source Odia language tools",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/soumendrak/openodia",
    author="Soumendra Kumar Sahoo",
    author_email="proud_odia@outlook.com",
    license="MIT",
    classifiers=[
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3 :: Only',
    ],
    packages=find_packages(exclude=["tests", "*.tests", "*.tests.*", "tests.*"]),
    include_package_data=True,
    install_requires=REQUIREMENTS,
    entry_points={"console_scripts": ["openodia=openodia.__main__:main"]},
)
